﻿using CommunityToolkit.Mvvm.ComponentModel;
using System.ComponentModel;
using StudiApp.Models;
using Syncfusion.Maui.Scheduler;
using System.Collections.ObjectModel;
using StudiApp.Repository;

namespace StudiApp.ViewModels;

public partial class TimetableViewModel : ObservableObject
{
    private const int WeekdayLength = 2;
    private const string SemesterFormatting = "yyyyMMddTHHmmssZ";
    private const int TodoBeforeDue = -20;
    
    [ObservableProperty]
    private ObservableCollection<SchedulerAppointment> _appointments;

    private IRepository<Lecture, int> _lectureRepository;
    private IRepository<ToDo, int> _todoRepository;

    public TimetableViewModel(IRepository<Lecture, int> lectureRepository, IRepository<ToDo, int> todoRepository)
    {
        _appointments = new ObservableCollection<SchedulerAppointment>();
        _lectureRepository = lectureRepository;
        _todoRepository = todoRepository;
        InsertMockData();
        FillAppointments();
    }

    #region Mocking - only for prototype

    private async void InsertMockData()
    {
        var bsys2Time = new DateTime(1, 1, 1, 10, 10, 0);
        var bsys2Time2 = new DateTime(1, 1, 1, 11, 5, 0);
        var we2Time = new DateTime(1, 1, 1, 15, 10, 0);
        await _lectureRepository.Create(MakeMockLecture("Bsys2", bsys2Time2));
        await _lectureRepository.Create(MakeMockLecture("Bsys2", bsys2Time));
        await _lectureRepository.Create(MakeMockLecture("WE2", we2Time));

        var bsysTodoDue = new DateTime(2023, 3, 31, 12, 00, 00);
        var weTodoDue = new DateTime(2023, 3, 27, 10, 00, 00);
        await _todoRepository.Create(MakeMockToDo(bsysTodoDue, "bsysTodo"));
        await _todoRepository.Create(MakeMockToDo(weTodoDue, "weTodo"));
    }

    private Lecture MakeMockLecture(string name, DateTime startTime)
    {
        return new Lecture
        {
            Id = 1, DayTime = new DayTime { DayOfWeek = 1, StartTime = startTime, EndTime = startTime.AddMinutes(45) },
            Name = name, 
        };
    }

    private ToDo MakeMockToDo(DateTime due, string name)
    {
        return new ToDo
        {
            Description = "TestTodo", Due = due, Name = name,
        };
    }

    #endregion
    

    private async void FillAppointments()
    {
        var lectures = await _lectureRepository.GetAll();
        foreach (var lecture in lectures)
        {
            Appointments.Add(AppointmentFromLecture(lecture));
        }

        var todos = await _todoRepository.GetAll();
        foreach (var todo in todos)
        {
            Appointments.Add(AppointmentFromTodo(todo));
        }
    }

    private SchedulerAppointment AppointmentFromLecture(Lecture lecture)
    {
        // test numbers, will come from semester later
        var semesterStart = new DateTime(2023, 1, 1);
        var semesterEnd = new DateTime(2023, 6, 1);
        
        var semesterEndStr = semesterEnd.ToString(SemesterFormatting);
        var weekday = (DayOfWeek)lecture.DayTime.DayOfWeek;
        var weekdayStr = weekday.ToString().Substring(0, WeekdayLength).ToUpper();

        var reccurenceRule = $"FREQ=WEEKLY;INTERVAL=1;UNTIL={semesterEndStr};BYDAY={weekdayStr}";

        var firstLecture = GetNextWeekday(semesterStart, weekday);
        var firstLectureStart = firstLecture.AddHours(lecture.DayTime.StartTime.Hour).AddMinutes(lecture.DayTime.StartTime.Minute);
        var firstLectureEnd = firstLecture.AddHours(lecture.DayTime.EndTime.Hour).AddMinutes(lecture.DayTime.EndTime.Minute);
        return new SchedulerAppointment()
        {
            Id = lecture.Id,
            StartTime = firstLectureStart,
            EndTime = firstLectureEnd,
            Subject = lecture.Name,
            RecurrenceRule = reccurenceRule,
        };
    }

    private SchedulerAppointment AppointmentFromTodo(ToDo todo)
    {
        return new SchedulerAppointment()
        {
            Id = todo.Id,
            StartTime = todo.Due.AddMinutes(TodoBeforeDue),
            EndTime = todo.Due,
            Subject = todo.Name
        };
    }

    private DateTime GetNextWeekday(DateTime date, DayOfWeek weekday)
    {
        var days = ((int) weekday - (int) date.DayOfWeek + 7) % 7;
        return date.AddDays(days);
    }
}
